package jdk.management.resource.internal.inst;

import jdk.internal.instrumentation.InstrumentationMethod;
import jdk.management.resource.ResourceRequest;
import jdk.management.resource.ResourceRequestDeniedException;
import java.io.IOException;
import jdk.management.resource.internal.ApproverGroup;
import jdk.management.resource.internal.ResourceIdImpl;
import java.io.FileDescriptor;
import jdk.internal.instrumentation.InstrumentationTarget;

@InstrumentationTarget("java.net.AbstractPlainSocketImpl")
abstract class AbstractPlainSocketImplRMHooks
{
    protected FileDescriptor fd;
    
    AbstractPlainSocketImplRMHooks() {
    }
    
    abstract void socketClose0(boolean p0);
    
    @InstrumentationMethod
    protected synchronized void create(boolean b) throws IOException {
        this.create(b);
        ResourceIdImpl of = ResourceIdImpl.of(this.fd);
        ResourceRequest approver = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd);
        long request = 0L;
        try {
            request = approver.request(1L, of);
            if (request < 1L) {
                this.socketClose0(false);
                throw new IOException("Resource limited: too many open file descriptors");
            }
        }
        catch (ResourceRequestDeniedException cause) {
            approver.request(-request, of);
            this.socketClose0(false);
            throw new IOException("Resource limited: too many open file descriptors", cause);
        }
        approver.request(-(request - 1L), of);
    }
    
    @InstrumentationMethod
    protected void close() throws IOException {
        ResourceIdImpl of = null;
        ResourceRequest approver = null;
        if (this.fd != null) {
            of = ResourceIdImpl.of(this.fd);
            approver = ApproverGroup.FILEDESCRIPTOR_OPEN_GROUP.getApprover(this.fd);
        }
        try {
            this.close();
        }
        finally {
            if (approver != null) {
                approver.request(-1L, of);
            }
        }
    }
}
